<?php
/**
 * Copyright (c) 2021, Nosto Solutions Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author Nosto Solutions Ltd <contact@nosto.com>
 * @copyright 2021 Nosto Solutions Ltd
 * @license http://opensource.org/licenses/BSD-3-Clause BSD 3-Clause
 *
 */

namespace Nosto\Tagging\Model\Service\Product;

use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\Product;
use Magento\Store\Model\Store;
use Nosto\Tagging\Helper\Data as NostoDataHelper;
use Nosto\Tagging\Util\Url as UrlUtil;

class ImageService
{
    /** @var Image  */
    private Image $imageHelper;

    /** @var NostoDataHelper  */
    private NostoDataHelper $nostoDataHelper;

    /**
     * ImageService constructor.
     * @param Image $imageHelper
     * @param NostoDataHelper $nostoDataHelper
     */
    public function __construct(
        Image $imageHelper,
        NostoDataHelper $nostoDataHelper
    ) {
        $this->imageHelper = $imageHelper;
        $this->nostoDataHelper = $nostoDataHelper;
    }

    /**
     * @param Product $product
     * @param Store $store
     * @return string|null
     */
    public function buildImageUrl(Product $product, Store $store)
    {
        $primary = $this->nostoDataHelper->getProductImageVersion($store);
        $secondary = 'image'; // The "base" image.
        $media = $product->getMediaAttributeValues();

        if (isset($media[$primary])) {
            $image = $media[$primary];
        } elseif (isset($media[$secondary])) {
            $image = $media[$secondary];
        } else {
            $image = $this->imageHelper->init($product, "product_base_image")->getUrl();
        }

        if (empty($image)) {
            return null;
        }

        return $this->finalizeImageUrl(
            $product->getMediaConfig()->getMediaUrl($image),
            $store
        );
    }

    /**
     * Finalizes product image urls, stips off "pub/" directory if applicable
     *
     * @param string $url
     * @param Store $store
     * @return string
     */
    public function finalizeImageUrl(string $url, Store $store)
    {
        if ($this->nostoDataHelper->getRemovePubDirectoryFromProductImageUrl($store)) {
            return (new UrlUtil())->removePubFromUrl($url);
        }

        return $url;
    }
}
